'==============================
' Initialize
'==============================
  Set wbemSvc = GetObject("winmgmts://" & "." & "/root/cimv2")
  Set fileSys = CreateObject("Scripting.FileSystemObject")
  windowsDir = FindWindowsDir

'==============================
' Main(Args[])
'==============================

  EnumerateDeviceDrivers
  PrintFibreChannel
  EnumerateProcessors
  EnumeratePhysicalMemory
  EnumerateDiskDrives
  EnumerateLogicalVolumes
  EnumerateDNSSettings
  EnumerateGateways

'==============================
' EnumerateDeviceDrivers
'==============================
Function EnumerateDeviceDrivers()

  Set driverSet = wbemSvc.ExecQuery("Select * from Win32_SystemDriver")

  index = 0
  For Each driver In driverSet
    PrintDriverInfo index,driver
    index = index + 1
  Next

  FindSpecialDrivers index

End Function

'==============================
' FindSpecialDrivers
'==============================
Function FindSpecialDrivers(index)

  index = FindSpecialDriver(index,"ibmspwdll","ibmspw.dll","rsa2","RSA II Daemon")
  index = FindSpecialDriver(index,"ibmsp6wdll","ibmsp6w.dll","ibmipmi","Mapping Layer for OSA IPMI")
  index = FindSpecialDriver(index,"ibmspw64dll","ibmspw64.dll","rsa2","RSA II Daemon - 64 bit")
  index = FindSpecialDriver(index,"ibmsp6w64dll","ibmsp6w64.dll","ibmipmi","Mapping Layer for OSA IPMI - 64 bit")
  index = FindSpecialDriver(index,"ibmsprdpexe","ibmsprdp.exe","null","RSA II Terminal Services Bridge")

End Function

'==============================
' FindSpecialDriver
'==============================
Function FindSpecialDriver(index, driverName, driverFile, subDir, description)

  FindSpecialDriver=index
' ### Search for the Driver in the following location order: ###
' ## C:\Windows\system32 
  pathName = windowsDir&"\system32\"&driverFile
  If fileSys.FileExists(pathName) = false Then
'   ## C:\Windows\system32\drivers
    pathName = windowsDir&"\system32\drivers\"&driverFile
    If fileSys.FileExists(pathName) = false Then
'     ## C:\Windows\system32\drivers\subdir
      pathName = windowsDir&"\system32\drivers\"&subdir&"\"&driverFile
      If fileSys.FileExists(pathName) = false Then
'       ## C:\Windows\syswow64
        pathName = windowsDir&"\syswow64\"&driverFile
        If fileSys.FileExists(pathName) = false Then
'         ## C:\Windows\syswow64\drivers
          pathName = windowsDir&"\syswow64\drivers\"&driverFile
          If fileSys.FileExists(pathName) = false Then
'           ## C:\Windows\syswow64\drivers\subdir
            pathName = windowsDir&"\syswow64\drivers\"&subdir&"\"&driverFile
          End If
        End If
      End If
    End If
  End If

  If fileSys.FileExists(pathName) Then
    PrintSpecialDriverInfo index,driverName,pathName,description
    FindSpecialDriver=index+1
  End If

End Function

'==============================
' PrintDriverInfo
'==============================
Function PrintDriverInfo(index, driver)

  WScript.StdOut.WriteLine "DeviceDriver." & index & ".Name=" & driver.Name
  WScript.StdOut.WriteLine "DeviceDriver." & index & ".Description=" & driver.Description
  WScript.StdOut.WriteLine "DeviceDriver." & index & ".PathName=" & driver.PathName
  WScript.StdOut.WriteLine "DeviceDriver." & index & ".ServiceType=" & driver.ServiceType
  WScript.StdOut.WriteLine "DeviceDriver." & index & ".StartMode=" & driver.StartMode
  WScript.StdOut.WriteLine "DeviceDriver." & index & ".Started=" & driver.Started

  PrintDriverFileInfo index,driver.PathName

End Function

'==============================
' PrintSpecialDriverInfo
'==============================
Function PrintSpecialDriverInfo(index, driverName, pathName, description)

  WScript.StdOut.WriteLine "DeviceDriver." & index & ".Name=" & driverName
  WScript.StdOut.WriteLine "DeviceDriver." & index & ".Description=" & description
  WScript.StdOut.WriteLine "DeviceDriver." & index & ".PathName=" & pathName
  WScript.StdOut.WriteLine "DeviceDriver." & index & ".ServiceType=Kernel Driver"
  WScript.StdOut.WriteLine "DeviceDriver." & index & ".StartMode=System"
  WScript.StdOut.WriteLine "DeviceDriver." & index & ".Started=True"

  PrintDriverFileInfo index,pathName

End Function

'==============================
' PrintDriverFileInfo
'==============================
Function PrintDriverFileInfo(index, pathName)

  pathName = FormatPathName(pathName)

  If Len(pathName) > 0 Then
    If fileSys.FileExists(pathName) Then
      Set fileObj = fileSys.GetFile(pathName)
      WScript.StdOut.WriteLine "DeviceDriver." & index & ".FileName=" & fileObj.Name
      WScript.StdOut.WriteLine "DeviceDriver." & index & ".FileType=" & fileObj.Type
      WScript.StdOut.WriteLine "DeviceDriver." & index & ".FileSize=" & fileObj.Size
      WScript.StdOut.WriteLine "DeviceDriver." & index & ".LastModified=" & fileObj.DateLastModified
      WScript.StdOut.WriteLine "DeviceDriver." & index & ".Version=" & fileSys.GetFileVersion(pathName)      
      WScript.StdOut.WriteLine "DeviceDriver." & index & ".SoftwareID=" & fileObj.Name
    End If
  End If 

End Function

'==============================
' FindWindowsDir
'==============================
Function FindWindowsDir()

  FindWindowsDir = "C:\WINDOWS"
  Set osSet = wbemSvc.ExecQuery("Select * from Win32_OperatingSystem")

  For Each osInst In osSet
    FindWindowsDir = osInst.WindowsDirectory
  Next

End Function

'==============================
' FormatPathName
'==============================
Function FormatPathName(pathName)

  If Len(pathName) > 0 Then
    pathName = Replace(pathName,"\??\","")
  End If 

  FormatPathName = pathName

End Function

'==============================
' PrintFibreChannel
'==============================
Function PrintFibreChannel()
  RunCommand "fcinfo.exe"
End Function

'==============================
' RunCommand
'==============================
Function RunCommand(cmdName)
  Set WshShell = CreateObject("WScript.Shell")
  On Error Resume Next
  Set WshExec = WshShell.Exec(cmdName)

  If Err.Number = 0 Then
    Do While Not WshExec.StdOut.AtEndOfStream
      WScript.StdOut.WriteLine WshExec.StdOut.ReadLine
    Loop
  End If
End Function

'==============================
' EnumerateProcessors
'==============================
Function EnumerateProcessors()

  On Error Resume Next
  Set procSet = wbemSvc.ExecQuery("Select * from Win32_Processor")

  index = 0
  For Each proc In procSet
    PrintProperty "Processor",index,"Name",proc.Name
    PrintProperty "Processor",index,"DeviceID",proc.DeviceID
    PrintProperty "Processor",index,"AddressWidth",proc.AddressWidth
    PrintProperty "Processor",index,"DataWidth",proc.DataWidth
    PrintProperty "Processor",index,"CurrentClockSpeed",proc.CurrentClockSpeed
    PrintProperty "Processor",index,"MaxClockSpeed",proc.MaxClockSpeed
    PrintProperty "Processor",index,"Family",proc.Family
    PrintProperty "Processor",index,"UniqueID",proc.ProcessorId
    PrintStateProperty "Processor",index,"HealthState",proc.HealthState
    index = index + 1
  Next

End Function

'==============================
' EnumeratePhysicalMemory
'==============================
Function EnumeratePhysicalMemory()

  On Error Resume Next
  Set memSet = wbemSvc.ExecQuery("Select * from Win32_PhysicalMemory")

  index = 0
  For Each mem In memSet
    PrintProperty "PhysicalMemory",index,"Name",mem.DeviceLocator
    PrintProperty "PhysicalMemory",index,"Tag",mem.Tag
    PrintProperty "PhysicalMemory",index,"Manufacturer",mem.Manufacturer
    PrintProperty "PhysicalMemory",index,"Model",mem.Model
    PrintProperty "PhysicalMemory",index,"SerialNumber",mem.SerialNumber
    PrintProperty "PhysicalMemory",index,"PartNumber",mem.PartNumber
    PrintProperty "PhysicalMemory",index,"FormFactor",mem.FormFactor
    PrintProperty "PhysicalMemory",index,"BankLabel",mem.BankLabel
    PrintProperty "PhysicalMemory",index,"Capacity",mem.Capacity
    PrintProperty "PhysicalMemory",index,"DataWidth",mem.DataWidth
    PrintProperty "PhysicalMemory",index,"InterleavePosition",mem.InterleavePosition
    PrintProperty "PhysicalMemory",index,"MemoryType",mem.MemoryType
    PrintProperty "PhysicalMemory",index,"PositionInRow",mem.PositionInRow
    PrintProperty "PhysicalMemory",index,"Speed",mem.Speed
    PrintProperty "PhysicalMemory",index,"TotalWidth",mem.TotalWidth
    index = index + 1
  Next

End Function

'==============================
' EnumerateDiskDrives
'==============================
Function EnumerateDiskDrives()

  On Error Resume Next
  Set driveSet = wbemSvc.ExecQuery("Select * from Win32_DiskDrive")

  index = 0
  For Each drive In driveSet
    If InStr(drive.Caption,"LSI") = 0 AND InStr(drive,drive.Caption,"Logical") = 0 Then
      PrintProperty "DiskDrive",index,"Name",drive.Caption
      PrintProperty "DiskDrive",index,"DeviceID",drive.DeviceID
      PrintProperty "DiskDrive",index,"DefaultBlockSize",drive.DefaultBlockSize
      PrintProperty "DiskDrive",index,"MaxBlockSize",drive.MaxBlockSize
      PrintProperty "DiskDrive",index,"MaxMediaSize",drive.MaxMediaSize
      PrintProperty "DiskDrive",index,"MinBlockSize",drive.MinBlockSize
      PrintProperty "DiskDrive",index,"NeedsCleaning",drive.NeedsCleaning
      PrintStateProperty "DiskDrive",index,"HealthState",proc.HealthState
      index = index + 1
    End If
  Next

End Function

'==============================
' EnumerateLogicalVolumes
'==============================
Function EnumerateLogicalVolumes()

  On Error Resume Next
  Set volumeSet = wbemSvc.ExecQuery("Select * from Win32_LogicalDisk where (DriveType=3 OR DriveType=5)")

  index = 0
  For Each volume In volumeSet
    PrintProperty "LogicalVolume",index,"Name",volume.Name
    PrintProperty "LogicalVolume",index,"DeviceID",volume.DeviceID
    PrintProperty "LogicalVolume",index,"BlockSize","1"
    PrintProperty "LogicalVolume",index,"ConsumableBlocks",volume.Size
    PrintProperty "LogicalVolume",index,"Capacity",volume.Size
    PrintProperty "LogicalVolume",index,"FreeSpace",volume.FreeSpace
    PrintProperty "LogicalVolume",index,"DataOrganization",volume.DataOrganization
    PrintProperty "LogicalVolume",index,"DataRedundancy",volume.DataRedundancy
    PrintProperty "LogicalVolume",index,"DeltaReservation",volume.DeltaReservation
    PrintProperty "LogicalVolume",index,"NumberOfBlocks",volume.Size
    PrintProperty "LogicalVolume",index,"PackageRedundancy",volume.PackageRedundancy
    PrintProperty "LogicalVolume",index,"SequentialAccess"",volume.SequentialAccess"
    index = index + 1
  Next

End Function

'==============================
' EnumerateDNSSettings
'==============================
Function EnumerateDNSSettings()

  On Error Resume Next
  Set dnsSet = wbemSvc.ExecQuery("Select * from Win32_NetworkAdapterConfiguration")

  index = 0
  For Each dns In dnsSet
    If dns.FullDNSRegistrationEnabled = "True" AND IsNull(dns.IPAddress) = "False" Then
     If InStr(dns.IPAddress(0),"0.0.0.0") = 0 Then
      PrintProperty "DNSSettings",index,"Name",dns.Description
      PrintProperty "DNSSettings",index,"SettingID",dns.SettingID
      PrintProperty "DNSSettings",index,"DomainName",dns.DNSDomain
      PrintProperty "DNSSettings",index,"RequestedHostname",dns.DNSHostName
      PrintProperty "DNSSettings",index,"RegisterThisConnectionsAddress",dns.FullDNSRegistrationEnabled
      PrintProperty "DNSSettings",index,"UseSuffixWhenRegistering",dns.DomainDNSRegistrationEnabled
      PrintArrayProperty "DNSSettings",index,"DNSServerAddresses",dns.DNSServerSearchOrder
      PrintArrayProperty "DNSSettings",index,"DNSDomainSuffixes",dns.DNSDomainSuffixSearchOrder
      index = index + 1
     End If
    End If
  Next

End Function

'==============================
' EnumerateGateways
'==============================
Function EnumerateGateways()

  On Error Resume Next
  Set dnsSet = wbemSvc.ExecQuery("Select * from Win32_NetworkAdapterConfiguration")

  index = 0
  For Each dns In dnsSet
    If dns.FullDNSRegistrationEnabled = "True" AND IsNull(dns.IPAddress) = "False" Then
     If InStr(dns.IPAddress(0),"0.0.0.0") = 0 Then
      PrintArrayProperty "Gateway",index,"Gateway",dns.DefaultIPGateway
      PrintArrayProperty "Gateway",index,"IPAddress",dns.IPAddress
      index = index + 1
     End If
    End If
  Next

End Function

'==============================
' PrintProperty
'==============================
Function PrintProperty(rscType, index, propName, propVal)

  If IsNull(propVal) = "False" AND Len(propVal) > 0 Then
    WScript.StdOut.WriteLine rscType & "." & index & "." & propName & "=" & propVal
  End If

End Function

'==============================
' PrintArrayProperty
'==============================
Function PrintArrayProperty(rscType, index, propName, propVal)

  index2 = 0
  If IsNull(propVal) = "False" AND IsArray(propVal) = "True" Then
    For Each x in propVal
      If index2 = 0 Then
        WScript.StdOut.Write rscType & "." & index & "." & propName & "=" & x
      Else
        WScript.StdOut.Write ";" & x
      End If
      index2 = index2 + 1
    Next
    If index2 > 0 Then
      Wscript.StdOut.WriteLine ""
    End If
  End If

End Function

'==============================
' PrintStateProperty
'==============================
Function PrintStateProperty(rscType, index, propName, propVal)

  If propVal = "OK" Then
    WScript.StdOut.WriteLine rscType & "." & index & "." & propName & "=1"
  End If
  If propVal = "Error" Then
    WScript.StdOut.WriteLine rscType & "." & index & "." & propName & "=3"
  End If

End Function

