#!/bin/sh

####################################
# printDiskPartitions
####################################
printDiskPartitions()
{
  count=0
  cat /proc/partitions | grep 'dasd\|sd\|hd' | while read ONE TWO THREE FOUR FIVE
    do
      tempName=`echo $FOUR | grep '[0123456789]'`
      if [ "$tempName"  = "" ]; then
        continue
      fi
      echo "DiskPartition.$count.Name=/dev/$FOUR"
      echo "DiskPartition.$count.DeviceID=/dev/$FOUR"
      echo "DiskPartition.$count.NumberOfBlocks=$THREE"
      echo "DiskPartition.$count.Capacity=$THREE"
      echo "DiskPartition.$count.BlockSize=1"
      echo "DiskPartition.$count.PartitionType=2"
      echo "DiskPartition.$count.HealthState=1"
      count=`expr $count + 1`
    done
  return
}

####################################
# printLogicalVolumes
####################################
printLogicalVolumes()
{
  count=0
  cat /proc/partitions | grep 'dasd\|sd\|' | while read ONE TWO THREE FOUR FIVE
    do
      tempName=`echo $FOUR | grep '[0123456789]'`
      if [ "$tempName"  = "" ]; then
        continue
      fi
      echo "LogicalVolume.$count.Name=/dev/$FOUR"
      echo "LogicalVolume.$count.DeviceID=/dev/$FOUR"
      echo "LogicalVolume.$count.NumberOfBlocks=$THREE"
      echo "LogicalVolume.$count.Capacity=$THREE"
      echo "LogicalVolume.$count.BlockSize=1"
      echo "LogicalVolume.$count.HealthState=1"
      count=`expr $count + 1`
    done
  return
}

####################################
# printzSeriesProcessors
####################################
printzSeriesProcessors()
{
  cat /proc/cpuinfo | grep 'processor ' | while read LINE
    do
      indexS=${LINE#*processor }
      cpuverS=${LINE#*version = }
      cpuuidS=${LINE#*identification = }
      cpumtmS=${LINE#*machine = }
      
      indexV=${indexS%%:*}
      cpuverV=${cpuverS%%,*}
      cpuuidV=${cpuuidS%%,*}
      cpumtmV=${cpumtmS%%,*}

      echo "Processor.$indexV.Name=proc$indexV"
      echo "Processor.$indexV.DeviceID=proc$indexV"
      echo "Processor.$indexV.UniqueID=$cpuuidV"
      echo "Processor.$indexV.Version=$cpuverV"
      echo "Processor.$indexV.AddressWidth=64"
      echo "Processor.$indexV.DataWidth=64"
      echo "Processor.$indexV.NumberOfEnabledCores=1"
      echo "Processor.$indexV.HealthState=1"
      echo "Processor.$indexV.OperatingState=8"
      echo "Processor.$indexV.Family=200"
    done
  return
}

####################################
# printPowerProcessors
####################################
printPowerProcessors()
{
  count=0
  for proc_dir in /proc/device-tree/cpus/Power*;
   do
     width=32
     name=CPU$count
     cpuid=${proc_dir#*@}
     if [ -f $proc_dir/name ]; then
       name=`cat $proc_dir/name`
     fi
     if [ -f $proc_dir/64-bit ]; then
       width=64
     fi

     echo "Processor.$count.Name=$name"
     echo "Processor.$count.DeviceID=CPU$count"
     echo "Processor.$count.Family=32"
     echo "Processor.$count.AddressWidth=$width"
     echo "Processor.$count.DataWidth=$width"
     echo "Processor.$count.UniqueID=$proc_dir/"
     echo "Processor.$count.Manufacturer=IBM"

     procDone=0
     procFound=0
     cat /proc/cpuinfo | while read TAG SEP VAL
       do
       if [ "$procDone" = "1" ]; then
         continue
       fi
       if [ "$TAG" = "processor" ]; then
         if [ "$procFound" = "1" ]; then procDone=1
         elif [ "$VAL" = "$cpuid" ]; then procFound=1
         fi
       fi
       if [ "$procFound" = "0" ]; then
         continue
       fi
       if [ "$TAG" = "cpu" ]; then
         echo "Processor.$count.Model=$VAL"
       fi
       if [ "$TAG" = "clock" ]; then
         clock=${VAL%.*}
         echo "Processor.$count.CurrentClockSpeed=$clock"
       fi
       if [ "$TAG" = "revision" ]; then
         echo "Processor.$count.Version=$VAL"
       fi
     done
     count=`expr $count + 1`
  done
  return
}

####################################
# printIntelProcessors
####################################
printIntelProcessors()
{
  if [ -f "./biosinfo_linux" ]; then
    ./biosinfo_linux 4 2>/dev/null
  fi
  return
}

####################################
# printIntelPhysicalMem
####################################
printIntelPhysicalMem()
{
  if [ -f "./biosinfo_linux" ]; then
    ./biosinfo_linux 6 2>/dev/null
  fi
  return
}

####################################
# printDiskDrives
####################################
printDiskDrives()
{
  count=0
  for dasd_dir in /sys/block/hd*;
   do
     if [ "$dasd_dir" = "/sys/block/hd*" ]; then
       continue
     fi
     dasd_status=0
     dasd_devid=${dasd_dir#/sys/block/}
     echo "DiskDrive.$count.Name=/dev/$dasd_devid"
     echo "DiskDrive.$count.DeviceID=/dev/$dasd_devid"
     echo "DiskDrive.$count.DeviceType=2"
     if [ -f $dasd_dir/size ]; then
       dasd_size=`cat $dasd_dir/size`
       dasd_size=`expr $dasd_size / 2`
       echo "DiskDrive.$count.Capacity=$dasd_size"
     fi
     echo "DiskDrive.$count.OperatingState=8"
     count=`expr $count + 1`
  done
  count=0
  for dasd_dir in /sys/block/sd*;
   do
     if [ "$dasd_dir" = "/sys/block/sd*" ]; then
       continue
     fi
     dasd_status=0
     dasd_devid=${dasd_dir#/sys/block/}
     echo "DiskDrive.$count.Name=/dev/$dasd_devid"
     echo "DiskDrive.$count.DeviceID=/dev/$dasd_devid"
     echo "DiskDrive.$count.DeviceType=2"
     if [ -f $dasd_dir/size ]; then
       dasd_size=`cat $dasd_dir/size`
       dasd_size=`expr $dasd_size / 2`
       echo "DiskDrive.$count.Capacity=$dasd_size"
     fi
     echo "DiskDrive.$count.OperatingState=8"
     count=`expr $count + 1`
  done
  return
}

####################################
# printzSeriesHardDrives
####################################
printzSeriesHardDrives()
{
  count=0
  for dasd_dir in /sys/bus/ccw/drivers/dasd*/*;
   do
     dasd_status=0
     dasd_subdir=${dasd_dir#/sys/bus/ccw/drivers/dasd*/}
     dasd_devid=hdisk${dasd_subdir#0.0.}
     echo "DiskDrive.$count.Name=$dasd_devid"
     echo "DiskDrive.$count.DeviceID=$dasd_devid"
     echo "DiskDrive.$count.DeviceType=2"
     if [ -f $dasd_dir/block/size ]; then
       dasd_size=`cat $dasd_dir/block/size`
       dasd_size=`expr $dasd_size / 2`
       echo "DiskDrive.$count.Capacity=$dasd_size"
     fi
     if [ -f $dasd_dir/online ]; then
       if [ `cat $dasd_dir/online` = "1" ]; then dasd_status=8
       else dasd_status=5
       fi
     fi
     echo "DiskDrive.$count.OperatingState=$dasd_status"
     count=`expr $count + 1`
  done
  return
}

####################################
# printDeviceDrivers
####################################
printDeviceDrivers()
{
  count=0
  lsmod | while read DRIVER_NAME THE_REST
    do
    if [ "$DRIVER_NAME"  = "Module" ]; then
      continue
    fi
    versionStr=""
    versionJustFound="false"
    for FIRST in `modinfo $DRIVER_NAME 2>/dev/null`; 
      do
        if [ "$versionJustFound" = "true" ]; then
          versionJustFound="false"
          versionStr=$FIRST
        fi
        if [ "$FIRST" = "version:" ]; then
          versionJustFound="true"
        fi
    done
    if [ "$versionStr" != "" ]; then
        filePart=`modinfo $DRIVER_NAME | grep 'filename:' 2>/dev/null`
        descPart=`modinfo $DRIVER_NAME | grep 'description:' 2>/dev/null`
        pathName=/${filePart#*/*}
        fileName=${pathName##*/}
        descript=${descPart#description:    *}
        version=${versionStr%[*}
        softwareId=`rpm -qf $pathName`
        if [ "${softwareId#file }" != "$softwareId" ]; then
          softwareId=""
        fi
        echo "DeviceDriver.$count.Name=$DRIVER_NAME"
        echo "DeviceDriver.$count.Description=$descript"
        echo "DeviceDriver.$count.PathName=$pathName"
        echo "DeviceDriver.$count.FileName=$fileName"
        echo "DeviceDriver.$count.Version=$version"
        echo "DeviceDriver.$count.SoftwareID=$softwareId"
        echo "DeviceDriver.$count.Started=True"
        count=`expr $count + 1`
    fi
  done
  return
}

####################################
# printDirectorAgent
####################################
printDirectorAgent()
{
  softwareName="IBM Director Core Services"
  softwareFile="/opt/ibm/director/version.lv1"
  if [ -f "/opt/ibm/director/version.key" ]; then
    softwareName="IBM Director Agent"
    softwareFile="/opt/ibm/director/version.key"
  fi
  if [ -f $sofwareFile ]; then
    echo "DirectorAgent.0.Name=$softwareName"
    echo "DirectorAgent.0.InstallPath=/opt/ibm/director"
    for LINE in `cat $softwareFile`; 
      do
        first=${LINE%*=*}
        second=${LINE#*=*}
        if [ "$first" = "version" ]; then
          echo "DirectorAgent.0.Version=$second"
        fi
        if [ "$first" = "builddate" ]; then
          echo "DirectorAgent.0.BuildDate=$second"
        fi
        if [ "$first" = "buildnum" ]; then
          echo "DirectorAgent.0.BuildNumber=$second"
        fi
    done    
  fi
  return
}

fcCount=0

####################################
# printFibreChannel
####################################
printFibreChannel()
{
  printQLogicFibreChannel
  printEmulexFibreChannel
  printLegacyQLogicFibreChannel
}

####################################
# printQLogicFibreChannel
####################################
printQLogicFibreChannel()
{
  for fc_dir in /sys/class/scsi_host/*;
   do
     fc_subdir=${fc_dir#/sys/class/scsi_host/}
     fc_dir2=/sys/class/fc_host/$fc_subdir
     ########################
     # QLogic - If the Model starts with QL then it is QLogic
     ########################
     if [ -f $fc_dir/model_name ]; then
       model=`cat $fc_dir/model_name`
       if [ ${model#QL} != $model ]; then
         echo "FCPort.$fcCount.Vendor=QLogic"
         echo "FCPort.$fcCount.Model=$model"
         if [ -f $fc_dir/serial_num ]; then
           serialNum=`cat $fc_dir/serial_num`
           echo "FCPort.$fcCount.SerialNumber=$serialNum"
         fi
         if [ -f $fc_dir/driver_version ]; then
           driverVersion=`cat $fc_dir/driver_version`
           echo "FCPort.$fcCount.DriverVersion=$driverVersion"
         fi
         if [ -f $fc_dir/fw_version ]; then
           firmwareVersion=`cat $fc_dir/fw_version`
           firmwareVersion=${firmwareVersion% *}
           echo "FCPort.$fcCount.FirmwareVersion=$firmwareVersion"
         fi
         if [ -f $fc_dir/unique_id ]; then
           uniqueId=`cat $fc_dir/unique_id`
           echo "FCPort.$fcCount.IOAddress=$uniqueId"
         fi
         if [ -f $fc_dir2/port_id ]; then
           portId=`cat $fc_dir2/port_id`
           portId=${portId#0x}
           echo "FCPort.$fcCount.DeviceID=$portId"
         fi
         if [ -f $fc_dir2/port_name ]; then
           portWWN=`cat $fc_dir2/port_name`
           portWWN=${portWWN#0x}
           echo "FCPort.$fcCount.PortWWN=$portWWN"
         fi
         if [ -f $fc_dir2/node_name ]; then
           nodeWWN=`cat $fc_dir2/node_name`
           nodeWWN=${nodeWWN#0x}
           echo "FCPort.$fcCount.NodeWWN=$nodeWWN"
         fi
         if [ -f $fc_dir2/fabric_name ]; then
           fabricName=`cat $fc_dir2/fabric_name`
           fabricName=${fabricName#0x}
           echo "FCPort.$fcCount.FabricName=$fabricName"
         fi
         if [ -f $fc_dir2/port_state ]; then
           portState=0
           if [ `cat $fc_dir2/port_state` = "Online" ]; then portState=8
           else portState=5
           fi
           echo "FCPort.$fcCount.OperatingState=$portState"
         fi
         if [ -f $fc_dir2/speed ]; then
           speedStr=`cat $fc_dir2/speed`
           if [ "${speedStr% Gbit}" != "$speedStr" ]; then
             speed=${speedStr% Gbit}000000000
             echo "FCPort.$fcCount.Speed=$speed"
           else
             if [ "${speedStr% Mbit}" != "$speedStr" ]; then
               speed=${speedStr% Mbit}000000
               echo "FCPort.$fcCount.Speed=$speed"
             fi
           fi
         fi
         fcCount=`expr $fcCount + 1`
       fi
     fi
  done
  return
}

####################################
# printEmulexFibreChannel
####################################
printEmulexFibreChannel()
{
  for fc_dir in /sys/class/scsi_host/*;
   do
     fc_subdir=${fc_dir#/sys/class/scsi_host/}
     fc_dir2=/sys/class/fc_host/$fc_subdir
     ########################
     # Emulex - If the Driver File contains Emulex then it is Emulex
     ########################
     if [ -f $fc_dir/lpfc_drvr_version ]; then
       driver=`cat $fc_dir/lpfc_drvr_version`
       if [ "${driver#Emulex}" != "$driver" ]; then
         echo "FCPort.$fcCount.Vendor=Emulex"
         if [ -f $fc_dir/modeldesc ]; then
           descr=`cat $fc_dir/modeldesc`
           echo "FCPort.$fcCount.Description=$descr"
         fi
         if [ -f $fc_dir/modelname ]; then
           model=`cat $fc_dir/modelname`
           echo "FCPort.$fcCount.Model=$model"
         fi
         if [ -f $fc_dir/serial_num ]; then
           serialNum=`cat $fc_dir/serial_num`
           echo "FCPort.$fcCount.SerialNumber=$serialNum"
         fi
         if [ -f $fc_dir/lpfc_drvr_version ]; then
           driverVersion=`cat $fc_dir/lpfc_drvr_version`
           driverVersion=${driverVersion#*driver }
           echo "FCPort.$fcCount.DriverVersion=$driverVersion"
         fi
         if [ -f $fc_dir/fwrev ]; then
           firmwareVersion=`cat $fc_dir/fwrev`
           firmwareVersion=${firmwareVersion%% *}
           echo "FCPort.$fcCount.FirmwareVersion=$firmwareVersion"
         fi
         if [ -f $fc_dir/unique_id ]; then
           uniqueId=`cat $fc_dir/unique_id`
           echo "FCPort.$fcCount.IOAddress=$uniqueId"
         fi
         if [ -f $fc_dir2/port_id ]; then
           portId=`cat $fc_dir2/port_id`
           portId=${portId#0x}
           echo "FCPort.$fcCount.DeviceID=$portId"
         fi
         if [ -f $fc_dir2/port_name ]; then
           portWWN=`cat $fc_dir2/port_name`
           portWWN=${portWWN#0x}
           echo "FCPort.$fcCount.PortWWN=$portWWN"
         fi
         if [ -f $fc_dir2/node_name ]; then
           nodeWWN=`cat $fc_dir2/node_name`
           nodeWWN=${nodeWWN#0x}
           echo "FCPort.$fcCount.NodeWWN=$nodeWWN"
         fi
         if [ -f $fc_dir2/fabric_name ]; then
           fabricName=`cat $fc_dir2/fabric_name`
           fabricName=${fabricName#0x}
           echo "FCPort.$fcCount.FabricName=$fabricName"
         fi
         if [ -f $fc_dir2/port_state ]; then
           portState=0
           if [ `cat $fc_dir2/port_state` = "Online" ]; then portState=8
           else portState=5
           fi
           echo "FCPort.$fcCount.OperatingState=$portState"
         fi
         if [ -f $fc_dir2/speed ]; then
           speedStr=`cat $fc_dir2/speed`
           if [ "${speedStr% Gbit}" != "$speedStr" ]; then
             speed=${speedStr% Gbit}000000000
             echo "FCPort.$fcCount.Speed=$speed"
           else
             if [ "${speedStr% Mbit}" != "$speedStr" ]; then
               speed=${speedStr% Mbit}000000
               echo "FCPort.$fcCount.Speed=$speed"
             fi
           fi
         fi
         fcCount=`expr $fcCount + 1`
       fi
     fi
  done
  return
}

####################################
# printLegacyQLogicFibreChannel
####################################
printLegacyQLogicFibreChannel()
{
  for fc_file in /proc/scsi/ql*/*;
   do
    if [ -f $fc_file ]; then
     full_file=`cat $fc_file`
     first_line=${full_file%%:*}
     sfw_line=`cat $fc_file | grep 'Firmware version'`
     serial_line=`cat $fc_file | grep 'Serial'`
     nodewwn_line=`cat $fc_file | grep 'adapter-node='`
     portwwn_line=`cat $fc_file | grep 'adapter-port='`
     state_line=`cat $fc_file | grep 'loop state'`

     if [ "${first_line##* QL}" != "$first_line" ]; then
        model=${first_line##* QL}
        model=QL${model%:}
     elif [ "${first_line##* QM}" != "$first_line" ]; then
        model=${first_line##* QM}
        model=QM${model%:}
     fi
     portId=00000${fc_file#/proc/scsi/ql*/}
     firmwareVersion=${sfw_line#*Firmware version }
     firmwareVersion=${firmwareVersion%% *}
     driverVersion=${sfw_line#*Driver version }
     driverVersion=${driverVersion%% *}
     serialNum=${serial_line#*Serial# }
     serialNum=${serialNum%% *}
     nodeWWN=${nodewwn_line#*adapter-node=}
     nodeWWN=${nodeWWN%%;*}
     portWWN=${portwwn_line#*adapter-port=}
     portWWN=${portWWN%%;*}

     if [ "$portWWN" != "" ]; then
       echo "FCPort.$fcCount.Vendor=QLogic"
       echo "FCPort.$fcCount.Model=$model"
       echo "FCPort.$fcCount.SerialNumber=$serialNum"
       echo "FCPort.$fcCount.DriverVersion=$driverVersion"
       echo "FCPort.$fcCount.FirmwareVersion=$firmwareVersion"
       echo "FCPort.$fcCount.DeviceID=$portId"
       echo "FCPort.$fcCount.PortWWN=$portWWN"
       echo "FCPort.$fcCount.NodeWWN=$nodeWWN"
       portState=5
       if [ "${state_line#*READY}" != "$state_line" ]; then
         portState=8
       fi
       echo "FCPort.$fcCount.OperatingState=$portState"
       fcCount=`expr $fcCount + 1`
     fi
    fi
  done
  return
}

####################################
# printGateways
####################################
printGateways()
{
  devicelist=`cat /proc/net/dev | grep ":" | tr -s ': ' ',,' | cut -d , -f 2`

  for netdevice in $devicelist
   do
    ipaddress=`ifconfig $netdevice | grep "inet addr" | tr -s ' ' ',' | cut -d , -f 3 | sed 's/addr://'`
    gateway=`route -n | grep "$netdevice" | grep "UG" | tr -s ' ' ',' | cut -d , -f 2`
    
    if [ "$gateway" != "" ] && [ "$ipaddress" != "" ]; then
      echo "Gateway.$fcCount.Gateway=$gateway"
      echo "Gateway.$fcCount.IPAddress=$ipaddress"
      fcCount=`expr $fcCount + 1`
    fi
  done
  return
}

####################################
# main(String[] args)
####################################
os_arch=`uname -m`
if [ "$os_arch" = "s390" ] || [ "$os_arch" = "s390x" ]; then
  printDiskPartitions
  printLogicalVolumes
  printzSeriesProcessors
  printzSeriesHardDrives
  printDirectorAgent
else
  if [ "$os_arch" = "ppc" ] || [ "$os_arch" = "ppc32" ] || [ "$os_arch" = "ppc64" ]; then
    printDiskDrives
    printDiskPartitions
    printLogicalVolumes
    printPowerProcessors
    printGateways
  else
    printDeviceDrivers
    printFibreChannel
    printDiskDrives
    printDiskPartitions
    printLogicalVolumes
    printIntelProcessors
    printIntelPhysicalMem
    printGateways
  fi
fi